@@ -1,10 +1,10 @@
-# RCS Status      : $Id: Font.pm,v 1.6 1999-03-07 16:00:53+01 jv Exp $
+# RCS Status      : $Id: Font.pm,v 1.23 2005-06-10 22:56:37+02 jv Exp jv $
 # Author          : Johan Vromans
-# Created On      : December 1999
+# Created On      : December 1998
 # Last Modified By: Johan Vromans
-# Last Modified On: Sun Mar  7 15:51:06 1999
-# Update Count    : 261
-# Status          : Looks okay
+# Last Modified On: Wed Dec  6 13:38:31 2006
+# Update Count    : 466
+# Status          : Released
 
 ################ Module Preamble ################
 
@@ -14,113 +14,60 @@ use strict;
 
 BEGIN { require 5.005; }
 
-use IO;
+use IO qw(File);
+use File::Spec;
+use PostScript::StandardEncoding;
+use PostScript::ISOLatin1Encoding;
 
 use vars qw($VERSION);
-$VERSION = "1.0";
+$VERSION = "1.04";
 
 # If you have the t1disasm program, have $t1disasm point to it.
 # This speeds up the glyph fetching.
 use vars qw($t1disasm);
 
-# Adobe StandardEncoding.
-my @StandardEncoding;
-my $StandardEncoding =
-  ".notdef .notdef .notdef .notdef .notdef .notdef .notdef .notdef ".
-  ".notdef .notdef .notdef .notdef .notdef .notdef .notdef .notdef ".
-  ".notdef .notdef .notdef .notdef .notdef .notdef .notdef .notdef ".
-  ".notdef .notdef .notdef .notdef .notdef .notdef .notdef .notdef ".
-  "space exclam quotedbl numbersign dollar percent ampersand quoteright ".
-  "parenleft parenright asterisk plus comma hyphen period slash zero ".
-  "one two three four five six seven eight nine colon semicolon less ".
-  "equal greater question at A B C D E F G H I J K L M N O P Q R S T U ".
-  "V W X Y Z bracketleft backslash bracketright asciicircum underscore ".
-  "quoteleft a b c d e f g h i j k l m n o p q r s t u v w x y z ".
-  "braceleft bar braceright asciitilde .notdef .notdef .notdef .notdef ".
-  ".notdef .notdef .notdef .notdef .notdef .notdef .notdef .notdef ".
-  ".notdef .notdef .notdef .notdef .notdef .notdef .notdef .notdef ".
-  ".notdef .notdef .notdef .notdef .notdef .notdef .notdef .notdef ".
-  ".notdef .notdef .notdef .notdef .notdef .notdef exclamdown cent ".
-  "sterling fraction yen florin section currency quotesingle ".
-  "quotedblleft guillemotleft guilsinglleft guilsinglright fi fl ".
-  ".notdef endash dagger daggerdbl periodcentered .notdef paragraph ".
-  "bullet quotesinglbase quotedblbase quotedblright guillemotright ".
-  "ellipsis perthousand .notdef questiondown .notdef grave acute ".
-  "circumflex tilde macron breve dotaccent dieresis .notdef ring ".
-  "cedilla .notdef hungarumlaut ogonek caron emdash .notdef .notdef ".
-  ".notdef .notdef .notdef .notdef .notdef .notdef .notdef .notdef ".
-  ".notdef .notdef .notdef .notdef .notdef .notdef AE .notdef ".
-  "ordfeminine .notdef .notdef .notdef .notdef Lslash Oslash OE ".
-  "ordmasculine .notdef .notdef .notdef .notdef .notdef ae .notdef ".
-  ".notdef .notdef dotlessi .notdef .notdef lslash oslash oe germandbls ".
-  ".notdef .notdef .notdef .notdef";
-
-# Adobe ISOLatin1Encoding.
-my @ISOLatin1Encoding;
-my $ISOLatin1Encoding =
-  ".notdef .notdef .notdef .notdef .notdef .notdef .notdef .notdef ".
-  ".notdef .notdef .notdef .notdef .notdef .notdef .notdef .notdef ".
-  ".notdef .notdef .notdef .notdef .notdef .notdef .notdef .notdef ".
-  ".notdef .notdef .notdef .notdef .notdef .notdef .notdef .notdef ".
-  "space exclam quotedbl numbersign dollar percent ampersand quoteright ".
-  "parenleft parenright asterisk plus comma minus period slash zero one ".
-  "two three four five six seven eight nine colon semicolon less equal ".
-  "greater question at A B C D E F G H I J K L M N O P Q R S T U V W X ".
-  "Y Z bracketleft backslash bracketright asciicircum underscore ".
-  "quoteleft a b c d e f g h i j k l m n o p q r s t u v w x y z ".
-  "braceleft bar braceright asciitilde .notdef .notdef .notdef .notdef ".
-  ".notdef .notdef .notdef .notdef .notdef .notdef .notdef .notdef ".
-  ".notdef .notdef .notdef .notdef .notdef dotlessi grave acute ".
-  "circumflex tilde macron breve dotaccent dieresis .notdef ring ".
-  "cedilla .notdef hungarumlaut ogonek caron space exclamdown cent ".
-  "sterling currency yen brokenbar section dieresis copyright ".
-  "ordfeminine guillemotleft logicalnot hyphen registered macron degree ".
-  "plusminus twosuperior threesuperior acute mu paragraph ".
-  "periodcentered cedilla onesuperior ordmasculine guillemotright ".
-  "onequarter onehalf threequarters questiondown Agrave Aacute ".
-  "Acircumflex Atilde Adieresis Aring AE Ccedilla Egrave Eacute ".
-  "Ecircumflex Edieresis Igrave Iacute Icircumflex Idieresis Eth Ntilde ".
-  "Ograve Oacute Ocircumflex Otilde Odieresis multiply Oslash Ugrave ".
-  "Uacute Ucircumflex Udieresis Yacute Thorn germandbls agrave aacute ".
-  "acircumflex atilde adieresis aring ae ccedilla egrave eacute ".
-  "ecircumflex edieresis igrave iacute icircumflex idieresis eth ntilde ".
-  "ograve oacute ocircumflex otilde odieresis divide oslash ugrave ".
-  "uacute ucircumflex udieresis yacute thorn ydieresis";
-
-my $trace;
-my $verbose;
-
 sub new {
     my $class = shift;
     my $font = shift;
     my (%atts) = (error => 'die',
 		  format => 'ascii',
-		  verbose => 0, trace => 0,
+		  verbose => 0, trace => 0, debug => 0,
 		  @_);
     my $self = { file => $font };
     bless $self, $class;
 
     return $self unless defined $font;
 
-    $trace = lc($atts{trace});
-    $verbose = $trace || lc($atts{verbose});
+    $self->{debug}   = $atts{debug};
+    $self->{trace}   = $self->{debug} || $atts{trace};
+    $self->{verbose} = $self->{trace} || $atts{verbose};
+
+    my $error = lc($atts{error});
+    $self->{die} = sub {
+	die(@_)     if $error eq "die";
+	warn(@_)    if $error eq "warn";
+    };
+
     $atts{format} = "ascii" if lc($atts{format}) eq "pfa";
     $atts{format} = "binary" if lc($atts{format}) eq "pfb";
 
+    $t1disasm = $self->_getexec ("t1disasm") unless defined $t1disasm;
+
     eval {
+
 	$self->_loadfont ();
 
 	# Reformat if needed.
 	$self->{format} = "ascii";
 	if ( lc($atts{format}) eq "asm" ) {
 	    print STDERR ($self->{file}, ": Converting to ASM format\n")
-	      if $verbose;
+	      if $self->{verbose};
 	    $self->{data} = $self->_pfa2asm;
 	    $self->{format} = "asm";
 	}
 	elsif ( lc($atts{format}) eq "binary" ) {
 	    print STDERR ($self->{file}, ": Converting to Binary format\n")
-	      if $verbose;
+	      if $self->{verbose};
 	    $self->{data} = $self->_pfa2pfb;
 	    $self->{format} = "binary";
 	}
@@ -128,21 +75,34 @@ sub new {
     };
 
     if ( $@ ) {
-	die ($@) unless lc($atts{error}) eq "warn";
-	warn ($@);
+	$self->_die($@);
 	return undef;
     }
 
     $self;
 }
 
-sub FileName	{ my $self = shift; $self->{file};    }
-sub FontName	{ my $self = shift; $self->{name};    }
-sub FontData	{ my $self = shift; ${$self->{data}}; }
-sub FamilyName	{ my $self = shift; $self->{family};  }
-sub FontType	{ my $self = shift; $self->{type};    }
-sub Version	{ my $self = shift; $self->{version}; }
-sub DataFormat  { my $self = shift; $self->{format};  }
+sub FileName	{ $_[0]->{file};      }
+sub FontName	{ $_[0]->{name};      }
+sub FamilyName	{ $_[0]->{family};    }
+sub FontType	{ $_[0]->{type};      }
+sub Version	{ $_[0]->{version};   }
+sub ItalicAngle	{ $_[0]->{italic};    }
+sub isFixedPitch{ $_[0]->{fixed};     }
+sub Weight	{ $_[0]->{weight};    }
+sub FontMatrix	{ $_[0]->{fontmatrix};}
+sub FontBBox	{ $_[0]->{fontbbox};  }
+sub DataFormat  { $_[0]->{dataformat} }
+
+sub FontData	{
+    my ($self, $format) = @_;
+    if ( defined $format ) {
+	# Not yet supported!
+	return ${$self->_cvtinternal($format)};
+    }
+    $self->{data} = $self->_cvtinternal unless exists $self->{data};
+    ${$self->{data}};
+}
 
 sub FontGlyphs {
     my $self = shift;
@@ -162,67 +122,97 @@ sub EncodingVector {
     return $enc if ref($enc) eq "ARRAY";
     # Return private copy for the standard encodings.
     if ( $enc eq "StandardEncoding" ) {
-	return StandardEncoding();
+	return scalar PostScript::StandardEncoding->array;
     }
     elsif ( $enc eq "ISOLatin1Encoding" ) {
-	return ISOLatin1Encoding();
+	return scalar PostScript::ISOLatin1Encoding->array;
     }
     undef;
 }
 
 sub StandardEncoding {
-    @StandardEncoding = split(' ', $StandardEncoding)
-      unless defined @StandardEncoding;
-    \@StandardEncoding;
+    return scalar PostScript::StandardEncoding->array;
 }
 
 sub ISOLatin1Encoding {
-    @ISOLatin1Encoding = split(' ', $ISOLatin1Encoding)
-      unless defined @ISOLatin1Encoding;
-    \@ISOLatin1Encoding;
+    return scalar PostScript::ISOLatin1Encoding->array;
 }
 
 sub _loadfont ($) {
 
     my $self = shift;
     my $data;			# font data
+    my $type;
 
     my $fn = $self->{file};
     my $fh = new IO::File;	# font file
     my $sz = -s $fn;	# file size
 
-    $fh->open ($fn) || die ("$fn: $!\n");
-    print STDERR ("$fn: Loading font file\n") if $verbose;
+    $fh->open ($fn) || $self->_die("$fn: $!\n");
+    print STDERR ("$fn: Loading font file\n") if $self->{verbose};
 
     # Read in the font data.
+    binmode($fh);		# (Some?) Windows need this
     my $len = 0;
     while ( $fh->sysread ($data, 32768, $len) > 0 ) {
 	$len = length ($data);
     }
     $fh->close;
-    print STDERR ("Read $len bytes from $fn\n") if $trace;
-    die ("$fn: Expecting $sz bytes, got $len bytes\n") unless $sz == $len;
+    $self->_die("$fn: Expecting at least 4 bytes, got ",
+		length($data), " bytes\n")
+      unless length($data) >= 4;
 
-    # Make ref.
-    $data = \"$data";		#";
+    $self->{dataformat} = 'ps';
+    if ( substr($data,0,4) eq "\0\1\0\0" || substr($data, 0, 4) eq "OTTO" ) {
+	# For the time being, Font::TTF is optional. Be careful.
+	eval {
+	    require PostScript::Font::TTtoType42;
+	};
+	if ( $@ ) {
+	    $self->_die("$fn: Cannot convert True Type font\n");
+	    return undef;
+	}
+	my $wrapper =
+	  new PostScript::Font::TTtoType42:: ($fn,
+					      verbose => $self->{verbose},
+					      trace   => $self->{trace},
+					      debug   => $self->{debug},
+					     );
+	$type = "t";
+	$data = $wrapper->as_string;
+	$self->{t42wrapper} = $wrapper;
+	$self->{dataformat} = "type42";
+	$self->{encoding} = "StandardEncoding";
+    }
+    else {
+	# Make ref.
+	$data = \"$data";		#";
+    }
+
+    print STDERR ("Read $len bytes from $fn\n") if $self->{trace};
+    $self->_die("$fn: Expecting $sz bytes, got $len bytes\n")
+      if $sz > 0 && $sz != $len;
 
     # Convert .pfb encoded font data.
     if ( $$data =~ /^\200[\001-\003]/ ) {
-	print STDERR ("$fn: Converting to ASCII format\n") if $verbose;
+	print STDERR ("$fn: Converting to ASCII format\n") if $self->{verbose};
 	$data = $self->_pfb2pfa ($data);
+	$self->{dataformat} = 'pfa';
     }
     # Otherwise, must be straight PostScript.
     elsif ( $$data !~ /^%!/ ) {
-	die ("$fn: Not a recognizable font file\n");
+	$self->_die("$fn: Not a recognizable font file\n");
+	return undef;
     }
 
     # Normalise line endings.
     $$data =~ s/\015\012?/\n/g;
 
     $self->{data} = $data;
+    $self->{type} = $type if defined $type;
 
     if ( $$data =~ /^%!FontType(\d+)\n\/(\S+)\n/ ) {
-	$self->{type} = $1;
+	$self->{type} = $1 unless defined $self->{type};
 	$self->{name} = $2;
     }
     elsif ( $$data =~ /\/FontName\s*\/(\S+)/ ) {
@@ -241,8 +231,34 @@ sub _loadfont ($) {
 	$self->{type} = $1 if $$data =~ /\/FontType\s+(\d+)/;
     }
     $self->{version} = $1 if $$data =~ /\/version\s*\(([^\051]+)\)/;
+    $self->{italic} = $1 if $$data =~ /\/ItalicAngle\s+([-+]?\d+)/;
+    $self->{fixed} = $1 eq "true"
+      if $$data =~ /\/isFixedPitch\s+(true|false)/;
+    if ( $$data =~ /\/Weight\s*\/(\S+)/ ) {
+	$self->{weight} = $1;
+    }
+    elsif ( $$data =~ /\/Weight\s*\(([^\051]+)\)/ ) {
+	$self->{weight} = $1;
+    }
+    if ( $$data =~ /\/FontMatrix\s*\[\s*(\d+(?:\.\d*)?)\s+(\d+(?:\.\d*)?)\s+(\d+(?:\.\d*)?)\s+(\d+(?:\.\d*)?)\s+(\d+(?:\.\d*)?)\s+(\d+(?:\.\d*)?)\s*\]/ ) {
+	$self->{fontmatrix} = [$1,$2,$3,$4,$5,$6];
+    }
+    if ( $$data =~ /\/FontBBox\s*\{\s*(-?\d+(?:\.\d*)?)\s+(-?\d+(?:\.\d*)?)\s+(-?\d+(?:\.\d*)?)\s+(-?\d+(?:\.\d*)?)\s*\}/ ) {
+	$self->{fontbbox} = [$1,$2,$3,$4];
+    }
+    $self;
+}
 
-$self;
+sub _qtfn ($) {
+    my $f = shift;
+    $f =~ s/([\\'])/'\\$1'/g;
+    "'".$f."'";
+}
+
+sub _cvtinternal {
+    my ($self, $format) = @_;
+    return $self->{data} if $format eq $self->{format};
+    undef;
 }
 
 sub _pfb2pfa ($;$) {
@@ -265,7 +281,7 @@ sub _pfb2pfa ($;$) {
     my $bin = "";		# accumulated unprocessed binary segments
     my $addbin = sub {		# binary segment processor
 	print STDERR ("Processing binary segment, ",
-		      length($bin), " bytes\n") if $trace;
+		      length($bin), " bytes\n") if $self->{trace};
 	($bin = uc (unpack ("H*", $bin))) =~ s/(.{64})/$1\n/g;
 	$newdata .= $bin;
 	$newdata .= "\n" unless $newdata =~ /\n$/;
@@ -278,7 +294,7 @@ sub _pfb2pfa ($;$) {
 	last if $$data =~ /^\200\003/; # EOF indicator
 
 	# Get font segment.
-	die ($self->{file}, ": Invalid font segment format\n")
+	$self->_die($self->{file}, ": Invalid font segment format\n")
 	  unless ($type, $info) = $$data =~ /^\200([\001-\002])(....)/s;
 
 	my $len = unpack ("V", $info);
@@ -290,12 +306,12 @@ sub _pfb2pfa ($;$) {
 	if ( ord($type) == 1 ) {	# ASCII segment
 	    $addbin->() if $bin ne "";
 	    print STDERR ($self->{file}, ": ASCII segment, $len bytes\n")
-	      if $trace;
+	      if $self->{trace};
 	    $newdata .= $seg;
 	}
 	else { # ord($type) == 2	# Binary segment
 	    print STDERR ($self->{file}, ": Binary segment, $len bytes\n")
-	      if $trace;
+	      if $self->{trace};
 	    $bin .= $seg;
 	}
     }
@@ -308,7 +324,7 @@ sub _pfa2pfb ($;$) {
 
     $data = $self->{data} unless defined $data;
 
-    return "\200\001".pack("V",length($data)).$data."\200\003"
+    return \do{"\200\001".pack("V",length($$data)).$$data."\200\003"}
       unless $$data =~ m{(^.*\beexec\s*\n+)
                          ([A-Fa-f0-9\n]+)
                          (\s*cleartomark.*$)}sx;
@@ -324,13 +340,10 @@ sub _pfa2pfb ($;$) {
 	$nulls -= 32;
     }
 
-    my $newdata = 
-      "\200\001".pack("V",length($pre)).$pre.
-      "\200\002".pack("V",length($bin)).$bin.
-      "\200\001".pack("V",length($post)).$post.
-      "\200\003";
-
-    return \$newdata;
+    return \do{"\200\001".pack("V",length($pre)).$pre.
+	       "\200\002".pack("V",length($bin)).$bin.
+	       "\200\001".pack("V",length($post)).$post.
+	       "\200\003" };
 }
 
 sub _pfa2asm ($;$) {
@@ -338,14 +351,20 @@ sub _pfa2asm ($;$) {
 
     $data = $self->{data} unless defined $data;
 
-    if ( defined $t1disasm ) {
-	my $fn = $self->{file};
-	$fn =~ s/[\\']/\\$1/g;
-	print STDERR ("+ $t1disasm '$fn'|\n") if $trace;
-	my $fh = new IO::File ("$t1disasm '$fn'|");
+    if ( $t1disasm ) {
+	my $fn = _qtfn($self->{file});
+	#### WARNING: This is Unix specific! ####
+	my $cmd = "$t1disasm $fn";
+	if ( $self->{type} eq 't' ) {
+	    $self->_die($self->{file},
+			": Logic error -- Cannot convert True Type font\n");
+	}
+
+	print STDERR ("+ $cmd |\n") if $self->{trace};
+	my $fh = new IO::File ("$cmd |");
 	local ($/);
 	my $newdata = <$fh>;
-	$fh->close or die ($self->{file}, ": $!");
+	$fh->close or warn ($cmd, ": return ". sprintf("%x", $?), "\n");
 	$newdata =~ s/\015\012?/\n/g;
 	return \$newdata;
     }
@@ -401,9 +420,17 @@ sub _getglyphnames ($;$) {
     my ($self, $data) = @_;
     my @glyphs = ();
 
+    print STDERR ($self->{file}, ": Getting glyph info\n") if $self->{verbose};
+
     $data = $self->{data} unless defined $data;
 
-    print STDERR ($self->{file}, ": Getting glyph info\n") if $verbose;
+    if ( $self->{dataformat} eq "type42" ) {
+	my $g = $self->{t42wrapper}->glyphnames;
+	print STDERR ($self->{file}, ": Number of glyphs = ",
+		      scalar(@$g), "\n")
+	  if $self->{verbose};
+	return $g;
+    }
 
     if ( $self->{format} eq "binary" ) {
 	$data = $self->_pfb2pfa ($data);
@@ -422,11 +449,16 @@ sub _getglyphnames ($;$) {
 	}
     }
 
-    while ( $data =~ m;((^\d*\s*/([.\w]+))|(\bend\b)).*\n;mg ) {
+#    while ( $data =~ m;((^\d*\s*/([.\w]+))|(\bend\b)).*\n;mg ) {
+    while ( $data =~ m;(/([.\w]+)\b|\bend\b);mg ) {
 	last if $1 eq "end";
-	push (@glyphs, $3);
+	push (@glyphs, $2);
     }
 
+    print STDERR ($self->{file}, ": Number of glyphs = ",
+		  scalar(@glyphs), "\n")
+      if $self->{verbose};
+
     \@glyphs;
 }
 
@@ -434,7 +466,8 @@ sub _getencoding ($;$) {
     my ($self, $data) = @_;
     my @glyphs = ();
 
-    print STDERR ($self->{file}, ": Getting encoding info\n") if $verbose;
+    print STDERR ($self->{file}, ": Getting encoding info\n")
+      if $self->{verbose};
 
     $data = $self->{data} unless defined $data;
     $data = $$data;		# deref
@@ -450,10 +483,10 @@ sub _getencoding ($;$) {
 	$enc =~ s/^\s+//;
 	$enc =~ s/\s+$//;
 	$enc =~ s/\s+/ /g;
-	if ( $enc eq $StandardEncoding ) {
+	if ( $enc eq PostScript::StandardEncoding->string ) {
 	    $enc = "StandardEncoding"
 	}
-	elsif ( $enc eq $ISOLatin1Encoding ) {
+	elsif ( $enc eq PostScript::ISOLatin1Encoding->string ) {
 	    $enc = "ISOLatin1Encoding"
 	}
 	else {
@@ -470,10 +503,10 @@ sub _getencoding ($;$) {
 	while ( $enc =~ m;dup (\d+)\s*/(\S+) put;g ) {
 	    $enc[$1] = $2;
 	}
-	if ( "@enc" eq $StandardEncoding ) {
+	if ( "@enc" eq PostScript::StandardEncoding->string ) {
 	    $enc = "StandardEncoding"
 	}
-	elsif ( "@enc" eq $ISOLatin1Encoding ) {
+	elsif ( "@enc" eq PostScript::ISOLatin1Encoding->string ) {
 	    $enc = "ISOLatin1Encoding"
 	}
 	else {
@@ -485,6 +518,33 @@ sub _getencoding ($;$) {
     undef;
 }
 
+sub _getexec ($) {
+    my ($self, $exec) = @_;
+    foreach ( File::Spec->path ) {
+	if ( -x "$_/$exec" ) {
+	    print STDERR ("Using $_/$exec\n") if $self->{verbose};
+	    return "$_/$exec";
+	}
+	elsif ( -x "$_/$exec.exe" ) {
+	    print STDERR ("Using $_/$exec.exe\n") if $self->{verbose};
+	    return "$_/$exec.exe";
+	}
+    }
+    ''
+}
+
+sub _die {
+    my ($self, @msg) = @_;
+    $self->{die}->(@msg);
+}
+
+# PostScript::TTtoType42 is actually an Font::TTF::Font object.
+# Font::TTF::Font uses cyclic structures, so we need this.
+sub DESTROY {
+    my $self = shift;
+    $self->{t42wrapper}->DESTROY if $self->{t42wrapper};
+}
+
 1;
 
 __END__
@@ -505,13 +565,17 @@ PostScript::Font - module to fetch data from PostScript fonts
 This package reads PostScript font files and stores the information in memory.
 
 Most font file formats that are in use are recognised, especially the
-Type1 and Type42 font formats. Other formats that usually parse okay
-are Type5 and Type3, although Type3 can sometimes fail depending on
+Type 1 and Type 42 font formats. Other formats that usually parse okay
+are Type 5 and Type 3, although Type 3 can sometimes fail depending on
 how weird the font information is stored.
 
 The input font file can be encoded in ASCII (so-called C<.pfa>
 format), or binary (so-called C<.pfb> format).
 
+If you have Martin Hosken's Font::TTF package installed,
+PostScript::Font can also handle True Type fonts. They are converted
+internally to Type 42 format.
+
 =head1 CONSTRUCTOR
 
 =over 4
@@ -526,9 +590,13 @@ The constructor will read the file and parse its contents.
 
 =over 4
 
-=item error => [ 'die' | 'warn' ]
+=item error => [ 'die' | 'warn' | 'ignore' ]
 
-How errors must be handled.
+B<DEPRECATED>. Please use 'eval { ... }' to intercept errors.
+
+How errors must be handled. Default is to call die().
+In any case, new() returns a undefined result.
+Setting 'error' to 'ignore' may cause surprising results.
 
 =item format => [ 'ascii' | 'pfa' | 'binary' | 'pfb' ]
 
@@ -543,6 +611,11 @@ Prints verbose info if I<value> is true.
 
 Prints tracing info if I<value> is true.
 
+=item debug => I<value>
+
+Prints debugging info if I<value> is true.
+Implies 'trace' and 'verbose'.
+
 =back
 
 =head1 INSTANCE METHODS
@@ -568,9 +641,36 @@ The family name of the font, e.g. 'Times'.
 
 The version of the font, e.g. '001.007'.
 
+=item ItalicAngle
+
+The italicity of the font, e.g. 0 (normal upright fonts) or -16 (italic font).
+
+=item isFixedPitch
+
+Indicates if this font has fixed pitch.
+
+=item Weight
+
+This font weight.
+
 =item FontType
 
-The font type, e.g. '1'.
+The font type, e.g. '1' for a Type 1 PostScript font, or 't' for a
+True Type font.
+
+True Type fonts will be converted to Type 42 internally, but still
+have 't' as FontType.
+
+=item FontMatrix
+
+The font matrix as a reference to an anonymous array with the 6 values.
+To find the font scale, use
+
+    int(1/$font->FontMatrix->[0])
+
+=item FontBBox
+
+The font bounding box as a reference to an anonymous array with the 4 values.
 
 =item DataFormat
 
@@ -579,7 +679,7 @@ The format in which the data is kept internally. See the B<format> option.
 =item FontData
 
 The complete contents of the file, normalised to Unix-style line endings.
-It is in the format as returned by the I<dataformat> method.
+It is in the format as returned by the I<DataFormat> method.
 
 =item Encoding
 
@@ -608,10 +708,12 @@ B<NOTE:> Getting the glyphs information can fail if the way it was
 stored in the font is not recognized by the parser. This is most
 likely to happen with manually constructed fonts.
 
-Extracting the glyphs can be slow. It can be speeded up by setting
-variable C<$PostScript::Font::t1disasm> to the I<t1disasm> program, if
-you have it. This does not apply to type 42 fonts, since these fonts
-do not require disassembly to get at the glyph list.
+Extracting the glyphs can be slow. It can be speeded up by using the
+external program I<t1disasm>. This program will be used automatically,
+if it can be found in the execution C<PATH>. Alternatively, you can
+set the variable C<$PostScript::Font::t1disasm> to point to the
+I<t1disasm> program. This does not apply to type 42 fonts, since these
+fonts do not require disassembly to get at the glyph list.
 
 =head1 CLASS METHODS
 
@@ -622,11 +724,56 @@ do not require disassembly to get at the glyph list.
 Returns a reference to an array that contains all the glyphs names for
 Adobe's Standard Encoding.
 
+If this is the only thing you want from this module, use
+C<PostScript::StandardEncoding> instead.
+
 =item ISOLatin1Encoding
 
 Returns a reference to an array that contains all the glyphs names for
 ISO-8859-1 (ISO Latin-1) encoding.
 
+If this is the only thing you want from this module, use
+C<PostScript::ISOLatin1Encoding> instead.
+
+=back
+
+=head1 EXTERNAL PROGRAMS
+
+I<t1disasm> can be used to speed up the fetching of the list of font
+glyphs from Type 1 fonts. It is called as follows:
+
+    t1disasm filename
+
+This invocation will write the disassembled version of the Type 1 font
+to standard output. I<t1disasm> is part of the I<t1utils> package that
+can be found at http://www.lcdf.org/~eddietwo/type/ and several other
+places.
+
+=head1 KNOWN BUGS
+
+Invoking external programs (like I<t1disasm>) is guaranteed to work on
+Unix only.
+
+=head1 SEE ALSO
+
+=over 4
+
+=item http://partners.adobe.com/asn/developer/PDFS/TN/T1_SPEC.PDF
+
+The specification of the Type 1 font format.
+
+=item http://partners.adobe.com/asn/developer/PDFS/TN/5012.Type42_Spec.pdf
+
+The specification of the Type 42 font format.
+
+=item http://fonts.apple.com/TTRefMan/index.html
+
+The True Type reference manual.
+
+=item http://partners.adobe.com/asn/developer/PDFS/TN/5004.AFM_Spec.pdf
+
+The specification of the Adobe font metrics file format.
+
 =back
 
 =head1 AUTHOR
@@ -635,7 +782,7 @@ Johan Vromans, Squirrel Consultancy <jvromans@squirrel.nl>
 
 =head1 COPYRIGHT and DISCLAIMER
 
-This program is Copyright 1993,1999 by Squirrel Consultancy. All
+This program is Copyright 2000,1998 by Squirrel Consultancy. All
 rights reserved.
 
 This program is free software; you can redistribute it and/or modify
